import path from 'path';
import express from 'express';
import cookieParser from 'cookie-parser';
import passport from 'passport';
import mongoose from 'mongoose';
import flash from 'connect-flash';
import session from 'express-session';
import MongoStore from 'connect-mongo';
import {DateTime} from 'luxon';

import passportConfig from './libs/passport';

// Import des routes
import globalRoutes from './routes';
import authRoutes from './routes/auth';

import {trustProxy, env, mongoDbUri, secret, port} from './config';

import {isXhr, getBaseUrl} from './helpers';
import createHttpError from 'http-errors';

passportConfig(passport);

mongoose.set('strictQuery', false);
mongoose
    .connect(mongoDbUri, {useNewUrlParser: true, useUnifiedTopology: true})
    .catch((err) => {
      console.log('Mongo err:', err);
      process.exit();
    });

const sess = {
  cookie: {
    maxAge: 86400000,
  },
  secret,
  saveUninitialized: false,
  resave: false,
  store: MongoStore.create({
    mongoUrl: mongoDbUri,
    mongoOptions: {useNewUrlParser: true, useUnifiedTopology: true},
  }),
};

const app = express();

app.set('trust proxy', trustProxy);
app.use(cookieParser());
app.use(flash());
app.use(express.json({limit: '50mb'}));
app.use(express.urlencoded({extended: false, limit: '50mb'}));

app.use(session(sess));

if (['production'].indexOf(env) !== -1) {
  sess.cookie.secure = true;
  /* eslint-disable func-names */
  app.use((req, res, next) => {
    if (req.secure) {
      next();
    } else {
      res.redirect(`https://${req.headers.host}${req.url}`);
    }
  });
}

app.use(passport.initialize());
app.use(passport.session());

app.set('views', path.join(__dirname, '../views'));
app.set('view engine', 'ejs');

app.use(express.static(path.join(__dirname, '../public')));
app.use('/uploads', express.static(path.join(__dirname, '../uploads')));

// Init des routes
app.use('/', globalRoutes);
app.use('/', authRoutes);

// Handle 404
app.use((req, res, next) => {
  next(createHttpError(404));
});

// Handle error
app.use((error, req, res, next) => {
  if (isXhr(req)) {
    const {message, errorCode, date} = error;
    res.status(error.errorCode || error.statusCode || 500).send({message, errorCode, date});
  } else {
    console.log('ERR:', error);
    res.status(error.errorCode || error.statusCode || 500);
    res.render('index', {
      pagename: '500',
      viewname: 'error',
      page: {
        title: error.message || '500: Oups… le serveur a crashé !',
      },
      baseUrl: getBaseUrl(req),
      params: {},
      query: {},
      user: req.user,
      error,
      DateTime,
    });
  }
});

console.log(`Server ready and listening on port ${port}!`);

export default app;
